-- -*-coding: utf-8;-*-

--[[
-- ************************************************************************
-- BuddyName.lua
-- ************************************************************************
-- This addon is released under GPL v3. All additionally used libraries are
-- released under the license that accompanies them.
--]]

BuddyName = LibStub("AceAddon-3.0"):NewAddon("BuddyName",
                                             "AceConsole-3.0")

local L = LibStub("AceLocale-3.0"):GetLocale("BuddyName")

-- Chat events that will be processed
local chatEvents = {
	 ["CHAT_MSG_CHANNEL"]        = false,
	 ["CHAT_MSG_SAY"]            = true,
	 ["CHAT_MSG_YELL"]           = true,
	 ["CHAT_MSG_WHISPER"]        = false, -- temporarily, due to bug #6
	 ["CHAT_MSG_GUILD"]          = true,
	 ["CHAT_MSG_OFFICER"]        = true,
	 ["CHAT_MSG_PARTY"]          = true,
	 ["CHAT_MSG_PARTY_LEADER"]   = true,
	 ["CHAT_MSG_RAID"]           = true,
	 ["CHAT_MSG_RAID_LEADER"]    = true,
}

local slashCommands = {
	 ["bn"]        = true,
	 ["buddyname"] = true,
	 ["BuddyName"] = true,
}

--[[ Local variables ]]
local BN_DB_Version = 2; -- Database format
local BN_CommVersion = 1; -- Message format communication protocol
local BN_Activated = false; -- Is this addon active? Modified by OnEnable and OnDisable

--[[ -- -------------------------------- -- ]]
--[[ -- Basic functions, no dependenices -- ]]
--[[ -- -------------------------------- -- ]]

--[[ Creates a new numeric timestamp for the current time and date
--   Returns:
--     1. Numeric: timestamp in format YYYYMMDDHHmm
--]]
function BuddyName:GetTimestamp()
	 local _,month,day,year = CalendarGetDate();
	 local hour,minute = GetGameTime();

	 local timestamp = minute + 100 * (hour + 100 * (day + 100 * (month + 100 * year)))

	 -- self:Print("Debug Timestamp: "..timestamp);

	 return timestamp
end

--[[ -- -------------------------- -- ]]
--[[ -- Database-related functions -- ]]
--[[ -- -------------------------- -- ]]

--[[ Transforms old DB entries to newer versions
--   Returns: nil
--]]
function BuddyName:UpdateDB()
	 if (self.db.realm.version==1) then
			self:Print(string.format(L["UPDATING_DB"], 2));

			-- perform updates to v2
			for p,v in pairs(self.db.realm.char) do
				 if (self.db.realm.char[p]) then
						self.db.realm.char[p] = {
							 buddy = v,
							 guild = "", -- TODO
							 timestamp = BuddyName:GetTimestamp(),
						}
				 end
			end

			self.db.realm.version=2
			self:Print(L["UPDATE_OK"])
	 end

	 --[[
	 if (self.db.realm.version==2) then
			self:Print(string.format(L["UPDATING_DB"], 3));
			-- perform update to v3 here
			self.db.realm.version=3
			self:Print(L["UPDATE_OK"])
	 end
	 ]]
end

--[[ Checks whether the given name is a valid player and has a buddy name in our DB
--   Returns:
--     1. Boolean: true if author was found in DB, false else
--     2. String: Buddy name of author, or nil if it doesn't exist or equals empty string
--]]
function BuddyName:IsInDB(author)
	 if (author~=nil and author~="") then
			-- if (UnitExists(author) and UnitIsPlayer(author)) then -- TODO: UnitExists returns only true if unit exists and is in current zone => remove
			if (self.db.realm.char and self.db.realm.char[author] and self.db.realm.char[author].buddy~="") then
				 return true,self.db.realm.char[author].buddy;
			end
			-- end
	 end
	 return false, nil
end

--[[ Adds a new entry to the DB or updates an existing one
--   Returns:
--     1. Boolean: true if successul, false else
--]]
function BuddyName:addToDB(charname, buddyname)
	 -- TODO: Check same things as in chatfilter: Is char really player, etc
	 BuddyName.db.realm.char[charname]= {
			buddy = buddyname,
			guild = "", -- TODO
			timestamp = BuddyName:GetTimestamp()
	 }
	 return true
end

--[[ Deletes an entry from the DB if it exists
--   Returns:
--     1. Boolean: true if deletion was successful, false else
--     2. String: buddy name if deletion was successful, nil else (e.g. char was not in DB)
--]]
function BuddyName:deleteFromDB(charname)
	 if (not BuddyName.db.realm.char[charname]) then
			return false,nil
	 end

	 local buddyname = BuddyName.db.realm.char[charname].buddy
	 BuddyName.db.realm.char[charname]=nil
	 return true,buddyname
end

--[[ Message filter, will modify author information.
--   Called automatically by WoW for registered chat events.
--   Returns:
--     1. Boolean: true, if message will be discarded, false else
--]]
local function ChatFilter(chatFrame, event, msg, author, ...)
	 if (chatEvents[event]==true) then -- really one of the filtered events (just in case)
			local replace,buddy = BuddyName:IsInDB(author) -- char in DB?
			if (replace) then
				 author=string.format("%s:%s", author, buddy)
			end
	 end
	 
	 return false, msg, author, ...
end

--[[ -- ----------------- -- ]]
--[[ -- AddOn interaction -- ]]
--[[ -- ----------------- -- ]]

--[[ Process slash commands (unexpected, eh?)
--   Returns: nil
--]]
function BuddyName:SlashProcessor(input)
	 -- 'input' contains whatever follows the slash command

	 -- split input string, ignore additional whitespace
	 local tbl = {}
	 for v in string.gmatch(input, "[^ ]+") do
			tinsert(tbl, v)
	 end

	 -- if no additional parameters => print out help text
	 if (#tbl == 0) then
			--self:Print(L["PLACEHOLDER"]);
			self:Print(L["HELP"])
			return;
	 end

	 local command = tremove(tbl,1)

	 -- Activate additional functionality
	 if (command=="on") then
			self:Enable()
			self:Print(L["ACTIVATED"]);
			return
	 end

	 -- Disable additional functionality
	 if (command=="off") then
			self:Disable()
			self:Print(L["DEACTIVATED"]);
			return
	 end

	 if (not BN_Active) then
			self:Print(L["NOT_ACTIVE"])
			return
	 end

	 -- add another person to the list
	 if (command=="add") then
			if (#tbl~=2) then
				 self:Print("Add: "..L["WRONG_NUMBER_OF_ARGUMENTS"])
			end
			-- TODO: Check same things as in chatfilter: Is char really player, etc
			BuddyName:addToDB(tbl[1], tbl[2])
			self:Print(string.format(L["BUDDY_ADD_OK"], tbl[1], tbl[2]));
			return
	 end

	 -- add another person to the list
	 if (command=="del" or command=="rm") then
			if (#tbl~=1) then
				 self:Print("Del: "..L["WRONG_NUMBER_OF_ARGUMENTS"])
			end
			local successful,buddyname = BuddyName:deleteFromDB(tbl[1])
			if (not successful) then
				 self:Print(string.format(L["BUDDY_DEL_ERROR"], tbl[1]))
				 return
			end
			self:Print(string.format(L["BUDDY_DEL_OK"], tbl[1], buddyname));
			return
	 end

	 -- list chars from database
	 if (command=="list") then
			self:Print("Buddy list:");

			for p,v in pairs(BuddyName.db.realm.char) do
				 local char = p;
				 local buddy = BuddyName.db.realm.char[p].buddy -- DB v2
				 if (buddy and buddy~="") then
						self:Print(string.format(L["BUDDY_LIST"], char, buddy));
				 end
			end

			-- TODO:
			-- Distinguish between all, guild, non-guild
			-- => Additional command arguments
			return
	 end

	 self:Print(string.format(L["UNKNOWN_COMMAND"], command));

-- debug
--	 for v = 1,(#tbl) do
--			self:Print(ChatFrame1, tbl[v]);
--	 end
end

--[[ -- --------------------------------------------- -- ]]
--[[ -- Basic functions automatically called by Ace3  -- ]]
--[[ -- --------------------------------------------- -- ]]

--[[ Executed only once on startup
--   Returns: nil
--]]
function BuddyName:OnInitialize()
	 --self:Print("OnInitialize called"); -- debug

	 -- Register Database
	 self.db = LibStub("AceDB-3.0"):New("BuddyName_DB")

	 -- Register Slash commands
	 for p,v in pairs(slashCommands) do
			if (v == true) then
				 self:RegisterChatCommand(p, "SlashProcessor")
			end
	 end
end


--[[ Executed each time the addon is switched on
--   Returns: nil
--]]
function BuddyName:OnEnable()
	 --self:Print("OnEnable called"); -- debug

	 if (BN_Active) then
			return -- we're already active
	 else
			BN_Active = true;
	 end

	 -- check if DB already exists, or if it is old and has to be converted
	 -- not done earlier because has to be called after PLAYER_ENTERING_WORLD (because of UpdateDB())
	 if (self.db.realm.version==nil) then -- if no version number, then either fresh db or very old. In each case, reset realm DB.
			self.db.realm.version=BN_DB_Version -- set current version number
			self.db.realm.char = {} -- set empty char table
	 elseif (self.db.realm.version < BN_DB_Version) then -- update old database
			BuddyName:UpdateDB();
	 end

	 -- activate chat filters
	 for p,v in pairs(chatEvents) do
			if (v == true) then
				 ChatFrame_AddMessageEventFilter(p, ChatFilter)
			end
	 end
end


--[[ Executed each time the addon is switched off
--   Returns: nil
--]]
function BuddyName:OnDisable()
	 --self:Print("OnDisable called");

	 if (not BN_Active) then
			return -- we're already inactive
	 else
			BN_Active = false;
	 end

	 -- remove chat filters
	 for p,v in pairs(chatEvents) do
			if (v == true) then
				 ChatFrame_RemoveMessageEventFilter(p, ChatFilter)
			end
	 end

end
